@@ -27,6 +27,8 @@ phenogram<-function(tree,x,fsize=1.0,ftype="reg",colors=NULL,axes=list(),add=FAL
 	else lwd<-2
 	if(hasArg(offset)) offset<-list(...)$offset
 	else offset<-0.2
+	if(hasArg(offsetFudge)) offsetFudge<-list(...)$offsetFudge
+	else offsetFudge<-1.37
 	if(hasArg(digits)) digits<-list(...)$digits
 	else digits<-2
 	if(hasArg(nticks)) nticks<-list(...)$nticks
@@ -36,7 +38,7 @@ phenogram<-function(tree,x,fsize=1.0,ftype="reg",colors=NULL,axes=list(),add=FAL
 	if(hasArg(spread.cost)) spread.cost<-list(...)$spread.cost
 	else spread.cost<-c(1,1)
 	if(hasArg(link)) link<-list(...)$link
-	else link<-0
+	else link<-if(spread.labels) 0.1*max(nodeHeights(tree)) else 0 
 	## end optional arguments
 	# check tree
 	if(class(tree)!="phylo") stop("tree should be an object of class 'phylo'")
@@ -57,8 +59,8 @@ phenogram<-function(tree,x,fsize=1.0,ftype="reg",colors=NULL,axes=list(),add=FAL
 	if(!is.null(axes$time)) xlim<-axes$time
 	if(!add&&is.null(xlim)){
 		pp<-par("pin")[1]
-		sw<-fsize*(max(strwidth(tree$tip.label,units="inches")))+offset*fsize*max(strwidth(tree$tip.label,units="inches"))/max(nchar(tree$tip.label))
-		alp<-optimize(function(a,H,link,sw,pp) (a*1.04*max(H)+link+sw-pp)^2,H=H,link,sw=sw,pp=pp,interval=c(0,1e6))$minimum
+		sw<-fsize*(max(strwidth(tree$tip.label,units="inches")))+offsetFudge*offset*fsize*strwidth("W",units="inches")
+		alp<-optimize(function(a,H,link,sw,pp) (a*1.04*(max(H)+link)+sw-pp)^2,H=H,link=link,sw=sw,pp=pp,interval=c(0,1e6))$minimum
 		xlim<-c(min(H),max(H)+link+sw/alp)
 	} 
 	if(is.null(tree$maps)){
@@ -68,7 +70,7 @@ phenogram<-function(tree,x,fsize=1.0,ftype="reg",colors=NULL,axes=list(),add=FAL
 			if(spread.labels) tt<-spreadlabels(tree,x[1:length(tree$tip)],fsize=fsize,cost=spread.cost) else tt<-x[1:length(tree$tip)]
 			if(tree$edge[1,2]<=length(tree$tip)){
 				if(fsize&&!add){
-					text(tree$tip.label[tree$edge[1,2]],x=H[1,2]+link,y=tt[tree$edge[1,2]],cex=fsize,font=ftype,pos=4,offset=offset+link)
+					text(tree$tip.label[tree$edge[1,2]],x=H[1,2]+link,y=tt[tree$edge[1,2]],cex=fsize,font=ftype,pos=4,offset=offset)
 					if(link>0) lines(x=c(H[1,2],H[1,2]+link),y=c(X[1,2],tt[tree$edge[1,2]]),lty=3)
 				}
 			}
@@ -78,7 +80,7 @@ phenogram<-function(tree,x,fsize=1.0,ftype="reg",colors=NULL,axes=list(),add=FAL
 			lines(H[i,],X[i,],type=type,lwd=lwd,lty=lty,col=colors)
 			if(tree$edge[i,2]<=length(tree$tip)){
 				if(fsize&&!add){ 
-					text(tree$tip.label[tree$edge[i,2]],x=H[i,2]+link,y=tt[tree$edge[i,2]],cex=fsize,font=ftype,pos=4,offset=offset+link)
+					text(tree$tip.label[tree$edge[i,2]],x=H[i,2]+link,y=tt[tree$edge[i,2]],cex=fsize,font=ftype,pos=4,offset=offset)
 					if(link>0) lines(x=c(H[i,2],H[i,2]+link),y=c(X[i,2],tt[tree$edge[i,2]]),lty=3)
 				}
 			}
@@ -102,7 +104,7 @@ phenogram<-function(tree,x,fsize=1.0,ftype="reg",colors=NULL,axes=list(),add=FAL
 			}
 			if(tree$edge[i,2]<=length(tree$tip)){
 				if(fsize&&!add){ 
-					text(tree$tip.label[tree$edge[i,2]],x=H[i,2]+link,y=tt[tree$edge[i,2]],cex=fsize,font=ftype,pos=4,offset=offset+link)
+					text(tree$tip.label[tree$edge[i,2]],x=H[i,2]+link,y=tt[tree$edge[i,2]],cex=fsize,font=ftype,pos=4,offset=offset)
 					if(link>0) lines(x=c(H[i,2],H[i,2]+link),y=c(X[i,2],tt[tree$edge[i,2]]),lty=3)
 				}
 			}
